"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Global data store object.  Not exactly "functional" programming, but I'm learning.
 *
 * @type {{}}
 */
var dataStore = {
    _instance: {},
    get instance() {
        var _this = this;
        if (!this._instance) {
            this._instance = {
                /**
                 * Just hold all the containers here.
                 */
                containers: {},
                /**
                 * Gets a container from the global data store.
                 *
                 * @param {string} name
                 * @param initializer
                 * @returns {IContainer}
                 */
                getContainer: function (name, initializer) {
                    // Ensure the container exists.
                    if (!dataStore.hasOwnProperty(name)) {
                        // If not, create it.
                        var store = {};
                        var container = { name: name, store: store, isInitialized: false };
                        // Initialize it.
                        if (initializer)
                            initializer(container);
                        _this.containers[name] = container;
                    }
                    // Then, return it.
                    return _this.containers[name];
                },
            };
        }
        return this._instance;
    },
};
//tslint:disable
exports.DataStore = dataStore.instance;
//tslint:enable
/**
 * Get a value from the data store.
 *
 * @param container
 * @param {string} key
 * @returns {any}
 */
exports.getContainerValue = function (container, key) {
    // If the data store has the requested key, return the associated value.
    if (container.store.hasOwnProperty(key))
        return container.store[key];
    // Return null instead?
    return undefined;
};
/**
 * Put a new value into the data store.  Overwrites are not allowed.
 *
 * @param container
 * @param {string} key
 * @param value
 * @returns {boolean}
 */
exports.putContainerValue = function (container, key, value) {
    // If the key already exists, return.
    if (container.store.hasOwnProperty(key))
        return false;
    // Otherwise, add the new key/value.
    container.store[key] = value;
    // Indicate success.
    return true;
};
/**
 * Update an existing value in the data store.  This only works on existing keys.
 *
 * @param container
 * @param {string} key
 * @param value
 * @returns {boolean}
 */
exports.updateContainerValue = function (container, key, value) {
    // If the key does not exist, return.
    if (!container.store.hasOwnProperty(key))
        return false;
    // Otherwise, update the value.
    container.store[key] = value;
    // Indicate success.
    return true;
};
